@@ -21,7 +21,7 @@
 
 /*
  * connection.c -- defines the connection object DBAPI 2.0
- * $Id: connection.c,v 1.47.2.4 2002/04/23 20:55:01 fog Exp $
+ * $Id: connection.c,v 1.47.2.5 2002/10/25 10:12:56 fog Exp $
  */
 
 #include <assert.h>
@@ -196,11 +196,11 @@ _psyco_conn_close(connobject *self)
     assert(self->cursors != NULL);
     len = PyList_Size(self->cursors);
     Dprintf("_psyco_conn_close(): len(self->cursors) = %d\n", len);
-    for (i = 0; i < len; i++) {
-        tmpobj = PyList_GetItem(self->cursors, 0);
+    for (i = len-1; i >= 0; i--) {
+        tmpobj = PyList_GetItem(self->cursors, i);
         assert(tmpobj);
         Py_INCREF(tmpobj);
-        PySequence_DelItem(self->cursors, 0);
+        PySequence_DelItem(self->cursors, i);
         ((cursobject *)tmpobj)->conn = NULL; /* orphaned */
         Dprintf("_psyco_conn_close(): cursor refcnt = %d\n",
                 tmpobj->ob_refcnt);
@@ -209,8 +209,8 @@ _psyco_conn_close(connobject *self)
     /* close all the open postgresql connections */
     len = PyList_Size(self->avail_conn);
     Dprintf("_psyco_conn_close(): len(self->avail_conn) = %d\n", len);
-    for (i = 0; i < len; i++) {
-        tmpobj = PyList_GetItem(self->avail_conn, 0);
+    for (i = len-1; i >= 0; i--) {
+        tmpobj = PyList_GetItem(self->avail_conn, i);
         assert(tmpobj);
         Py_INCREF(tmpobj);
         if ((keeper = (connkeeper *)PyCObject_AsVoidPtr(tmpobj))) {
@@ -222,7 +222,7 @@ _psyco_conn_close(connobject *self)
                 free(keeper);
             }
         }
-        PySequence_DelItem(self->avail_conn, 0);
+        PySequence_DelItem(self->avail_conn, i);
         Py_DECREF(tmpobj);
     }
     
