@@ -21,7 +21,7 @@
 
 /*
  * cursor.c - defines the cursor object DBAPI 2.0
- * $Id: cursor.c,v 1.102.2.7 2002/06/17 23:45:00 fog Exp $
+ * $Id: cursor.c,v 1.102.2.12 2002/10/25 10:12:56 fog Exp $
  */
 
 #include <assert.h>
@@ -239,10 +239,6 @@ pgconn_resolve_critical(cursobject *self)
     if (self->critical) {
         Dprintf("pgconn_resolve_critical(): error = %s\n", self->critical);
         PyErr_SetString(OperationalError, self->critical);
-        /* we don't want to destroy this cursor but just to close it, so we
-           incref it to prevent the garbage collector to invoke the destroy
-           function */
-        /* Py_INCREF(self); */
         psyco_curs_close(self, NULL);
         Dprintf("pgconn_resolve_critical(): resolved\n");
     }
@@ -675,7 +671,6 @@ psyco_curs_abort(cursobject *self, PyObject *args) {
     return result;
 }
 
-
 /* psyco_curs_commit() - execute a commit
  *
  * this function locks the keeper
@@ -708,8 +703,6 @@ psyco_curs_commit(cursobject *self, PyObject *args) {
 }
 
 
-
-
 /* _psyco_curs_execute() - execute a query and parse results, used by both the
    .execute() and the .callproc() methods */
 static PyObject *
@@ -758,25 +751,18 @@ _psyco_curs_execute(cursobject *self, char *query)
 
     pgstatus = PQresultStatus(self->pgres);
 
-    /* error! error! */
-    if (pgstatus != PGRES_TUPLES_OK && pgstatus != PGRES_COMMAND_OK) {
-        char *pgerr = PQresultErrorMessage(self->pgres);
-        char *errstr = NULL;
-        
-        if (asprintf(&errstr, "%s\n%s", pgerr, query) >= 0) {
-            PyErr_SetString(ProgrammingError, errstr);
-            free(errstr);
-        }
-        else {
-            PyErr_SetString(ProgrammingError, pgerr);
-        }
-        CLEARPGRES(self->pgres);
-        goto error;
-    }
+    switch(pgstatus) {
 
-    /* the query was a select, we got some tuples */
-    else if (pgstatus == PGRES_TUPLES_OK)
-    {
+        /* send data to the backend */
+    case PGRES_COPY_OUT:
+        break;
+
+        /* data from the backend */
+    case PGRES_COPY_IN:
+        break;
+
+        /* tuples, this was a select */
+    case PGRES_TUPLES_OK: {
         int i, pgnfields = PQnfields(self->pgres);
         int pgbintuples = PQbinaryTuples(self->pgres);
         
@@ -798,10 +784,10 @@ _psyco_curs_execute(cursobject *self, char *query)
             PyObject *cast;
             
             PyTuple_SET_ITEM(self->description, i, dtitem);
-
-            /* fill the right cast function by accessing the global dictionary
-               of casting objects.  If we got no defined cast use the default
-               one.
+            
+            /* fill the right cast function by accessing the global
+               dictionary of casting objects.  If we got no defined cast
+               use the default one.
             */
             if (!(cast = PyDict_GetItem(psyco_types, type))) {
                 Dprintf("_psyco_curs_execute: cast %d not found, using "
@@ -812,49 +798,52 @@ _psyco_curs_execute(cursobject *self, char *query)
                is binary use the default cast.
             */
             else if (pgbintuples && cast == psyco_binary_cast) {
-                Dprintf("_psyco_curs_execute: Binary cursor and binary field: "
-                        "%i using default cast\n", PQftype(self->pgres,i));
-                cast = psyco_default_cast;
+                Dprintf("_psyco_curs_execute: Binary cursor and "
+                        "binary field: %i using default cast\n",
+                        PQftype(self->pgres,i));
+                    cast = psyco_default_cast;
             }
             Dprintf("_psyco_curs_execute: using cast at %p for type %d\n",
                     cast, PQftype(self->pgres,i));
             Py_INCREF(cast);
             PyTuple_SET_ITEM(self->casts, i, cast);
-            
+                
             /* fill the other fields */
             PyTuple_SET_ITEM(dtitem, 0,
                              PyString_FromString(PQfname(self->pgres,i)));
             PyTuple_SET_ITEM(dtitem, 1, type);
             
-            /* FIXME: display size is the maximum size of this field in result
-               tuples? what should we do with it? */
+            /* FIXME: display size is the maximum size of this field in
+               result tuples? what should we do with it? */
             /* for (j = 0; j < self->rowcount; j++) {
-                len = PQgetlength(self->pgres, j, i);
-                maxl = ( maxl > len ? maxl : len);
-            }
-            PyTuple_SET_ITEM(dtitem, 2, PyInt_FromLong(maxl)); */
+               len = PQgetlength(self->pgres, j, i);
+               maxl = ( maxl > len ? maxl : len);
+               }
+               PyTuple_SET_ITEM(dtitem, 2, PyInt_FromLong(maxl)); */
             Py_INCREF(Py_None);
             PyTuple_SET_ITEM(dtitem, 2, Py_None);
             
             /* size on the backend */
             PyTuple_SET_ITEM(dtitem, 3,
                              PyInt_FromLong(PQfsize(self->pgres, i)));
-
+            
             /* FIXME: scale??? */
             Py_INCREF(Py_None);
             PyTuple_SET_ITEM(dtitem, 4, Py_None);
-
+            
             /* FIXME: precision??? */
             Py_INCREF(Py_None);
             PyTuple_SET_ITEM(dtitem, 5, Py_None);
-
+            
             /* FIXME: null_ok??? */
             Py_INCREF(Py_None);
             PyTuple_SET_ITEM(dtitem, 6, Py_None);
         }
     }
-    
-    else {  /* PGRES_COMMAND_OK */
+        break;
+
+        /* ok but no tuples */
+    case PGRES_COMMAND_OK:
         Dprintf("_psyco_curs_execute(): command returned OK (no tuples)\n");
         /* FIXME: why we read a value that we _know_ being 0? */
         self->rowcount = atoi(PQcmdTuples(self->pgres));
@@ -866,8 +855,31 @@ _psyco_curs_execute(cursobject *self, char *query)
         Py_XDECREF(self->casts);
         self->casts = NULL;
         self->notuples = 1;
+        break;
+        
+            /* error! error! */
+    default: {
+        char *pgerr = PQresultErrorMessage(self->pgres);
+        char *errstr = NULL;
+        
+        if (asprintf(&errstr, "%s\n%s", pgerr, query) >= 0) {
+            if (!strncmp(pgerr, "ERROR:  Cannot insert a duplicate key", 37)
+                || !strncmp(pgerr, "ERROR:  ExecAppend: Fail to add null", 36)
+                || !strstr(pgerr, "referential integrity violation"))
+                PyErr_SetString(IntegrityError, errstr);
+            else
+                PyErr_SetString(ProgrammingError, errstr);
+            free(errstr);
+        }
+        else {
+            PyErr_SetString(ProgrammingError, pgerr);
+        }
+        CLEARPGRES(self->pgres);
+        goto error;
     }
-
+        break;
+    }
+    
     /* here we replace status with its default value */
     pthread_mutex_lock(&(self->keeper->lock));
     self->keeper->status = old_keeper_status;
@@ -875,18 +887,16 @@ _psyco_curs_execute(cursobject *self, char *query)
     Py_INCREF(Py_None);
     return Py_None;
     
- error:
-    Dprintf("_psyco_curs_execute(): error, resetting connection\n");
+  error:
+    Dprintf("_psyco_curs_execute(): error, NOT resetting connection\n");
     pthread_mutex_lock(&(self->keeper->lock));
-    Py_BEGIN_ALLOW_THREADS;
     self->keeper->status = old_keeper_status;
-    abort_pgconn(self);
     pthread_mutex_unlock(&(self->keeper->lock));
-    Py_END_ALLOW_THREADS;
     EXC_IFCRITICAL(self);
     return NULL;    
 }
 
+
 /* _mogrify_dict() _mogrify_sequence() - used to implement string quoting */
 
 static char *
@@ -960,15 +970,16 @@ _mogrify_seq(PyObject *seq, PyObject *fmt)
         if (PyString_Check(value)) {
             PyTuple_SET_ITEM(new_seq, i, new_psyco_quotedstringobject(value));
             s = _mogrify_fmt(s, 0);
+            Py_DECREF(value);
         }
         else if (value == Py_None) {
             PyTuple_SET_ITEM(new_seq, i, PyString_FromString("NULL"));
             Py_DECREF(Py_None);
             s = _mogrify_fmt(s, 's');
+            Py_DECREF(value);
         }
         else {
             PyTuple_SET_ITEM(new_seq, i, value);
-            Py_INCREF(value);
             s = _mogrify_fmt(s, 0);
         }
     }
@@ -1002,15 +1013,72 @@ psyco_curs_execute(cursobject *self, PyObject *args)
        we do it using python methods */
     if (d)
     {
-        if (PyDict_Check(d)) { d = _mogrify_dict(d, operation); }
-        else if (PySequence_Check(d)) { d = _mogrify_seq(d, operation); }
+        if (PyDict_Check(d)) {
+            d = _mogrify_dict(d, operation);
+        }
+        else if (PySequence_Check(d)) {
+            d = _mogrify_seq(d, operation);
+        }
         else {
             PyErr_SetString(PyExc_TypeError,
                         "execute, argument 2: expected sequence or mapping");
             return NULL;
         }
-        
-        if (!(pystr = PyString_Format(operation, d))) return NULL;
+
+        /* if PyString_Format() return NULL an error occured: if the error is
+           a TypeError we need to check the exception.args[0] string for the
+           values:
+           
+                   "not enough arguments for format string"
+                   "not all arguments converted"
+
+           and return the appropriate ProgrammingError. we do that by grabbing
+           the curren exception (we will later restore it if the type or the
+           strings do not match. */
+        if (!(pystr = PyString_Format(operation, d))) {
+            PyObject *err, *arg, *trace;
+            int pe = 0;
+
+            Dprintf("psyco_curs_execute(): PyString_Format() error\n");
+
+            PyErr_Fetch(&err, &arg, &trace);
+            
+            if (err && PyErr_GivenExceptionMatches(err, PyExc_TypeError)) {
+                Dprintf("psyco_curs_execute(): TypeError exception catched\n");
+                
+                PyErr_NormalizeException(&err, &arg, &trace);
+                Dprintf("psyco_curs_execute(): exception normalized\n");
+                
+                if (PyObject_HasAttrString(arg, "args")) {
+                    PyObject *args = PyObject_GetAttrString(arg, "args");
+                    PyObject *str = PySequence_GetItem(args, 0);
+                    char *s = PyString_AS_STRING(str);
+
+                    Dprintf("psyco_curs_execute(): s = %s\n", s);
+
+                    if (!strcmp(s, "not enough arguments for format string")
+                      || !strcmp(s, "not all arguments converted")) {
+                        Dprintf("psyco_curs_execute(): exception matches\n");
+                        PyErr_SetString(ProgrammingError, s);
+                        pe = 1;
+                        Dprintf("psyco_curs_execute(): new exception set\n");
+                    }
+
+                    Py_DECREF(args);
+                    Py_DECREF(str);
+                    Dprintf("psyco_curs_execute(): arguments destroyed\n");
+                }
+            }
+
+            /* if we did not manage our own exception, restore old one */
+            if (pe == 1) {
+                Py_XDECREF(err); Py_XDECREF(arg); Py_XDECREF(trace);
+            }
+            else {
+                PyErr_Restore(err, arg, trace);
+            }
+            return NULL;
+        }
         query = strdup(PyString_AsString(pystr));
 
         Dprintf("psyco_curs_execute(): d->refcnt = %d\n", d->ob_refcnt);
@@ -1021,6 +1089,7 @@ psyco_curs_execute(cursobject *self, PyObject *args)
     else {
         query = strdup(PyString_AsString(operation));
     }
+
     Dprintf("psyco_curs_execute(): operation->refcnt = %d\n",
             operation->ob_refcnt);
 
@@ -1222,9 +1291,12 @@ psyco_curs_fetchone(cursobject *self, PyObject *args)
             Dprintf("psyco_curs_fetchone: row %ld, element %d, len %i\n",
                     self->row, i, l);
         }
-        
+
+        Dprintf("psyco_curs_fetchone: str->refcnt = %d\n", str->ob_refcnt);
         PyTuple_SET_ITEM(arg, 0, str);
         val = PyObject_CallObject(PyTuple_GET_ITEM(self->casts, i), arg);    
+        Dprintf("psyco_curs_fetchone: val->refcnt = %d\n", val->ob_refcnt);
+        Dprintf("psyco_curs_fetchone: str->refcnt = %d\n", str->ob_refcnt);
         Py_DECREF(arg);
         
         if (val) {
@@ -1377,8 +1449,6 @@ psyco_curs_dictfetchone(cursobject *self, PyObject *args)
     return dict;
 }
 
-/* cosimo 888268 */
-
 /* psyco_curs_dictfetchmany() - fetch many rows into dictionaries */
    
 static char psyco_curs_dictfetchmany__doc__[] = 
