@@ -20,7 +20,7 @@
 
 /*
  * typeobj.c -- define the type objects
- * $Id: typeobj.c,v 1.20.2.10 2002/06/17 23:45:00 fog Exp $
+ * $Id: typeobj.c,v 1.20.2.14 2002/10/25 10:12:56 fog Exp $
  */
 
 #include "assert.h"
@@ -34,7 +34,7 @@ static PyObject *
 psyco_INTEGER_cast(PyObject *s)
 {
     PyObject *i;
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
     i = PyNumber_Int(s);
     Dprintf("psyco_STRING_cast(): i->refcnt = %d\n", i->ob_refcnt);
     return i;
@@ -43,21 +43,21 @@ psyco_INTEGER_cast(PyObject *s)
 static PyObject *
 psyco_LONGINTEGER_cast(PyObject *s)
 {
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
     return PyNumber_Long(s);
 }
 
 static PyObject *
 psyco_FLOAT_cast(PyObject *s)
 {
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
     return PyNumber_Float(s);
 }
 
 static PyObject *
 psyco_STRING_cast(PyObject *s)
 {
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
     Py_INCREF(s);
     return s;
 }
@@ -69,17 +69,18 @@ psyco_DATE_cast(PyObject *s)
     int hh=0, mm=0;
     double ss=0.0;
     
-    char *str = PyString_AsString(s);
+    char *str;
 
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
+    str = PyString_AsString(s);
     
     /* check for infinity */
     if (!strcmp(str, "infinity") || !strcmp(str, "-infinity")) {
         if (str[0] == '-') {
-            return mxDateTimeP->DateTime_FromDateAndTime(-5851455,1,1, 0,0,0);
+            return mxDateTimeP->DateTime_FromDateAndTime(-999998,1,1, 0,0,0);
         }
         else {
-            return mxDateTimeP->DateTime_FromDateAndTime(5867440,12,31, 0,0,0);
+            return mxDateTimeP->DateTime_FromDateAndTime(999999,12,31, 0,0,0);
         }
     }
     
@@ -94,52 +95,16 @@ psyco_DATE_cast(PyObject *s)
     return mxDateTimeP->DateTime_FromDateAndTime(y, m, d, hh, mm, ss);
 }
 
-#ifdef UNDEF
-static PyObject *
-psyco_TIME_cast(PyObject *s)
-{
-    time_t now;
-    struct tm tm, *today;
-    int n, hh=0, mm=0;
-    double ss=0.0;
-    
-    char *str = PyString_AsString(s);
-    
-    /* postgres gives us only the time but a DateTime object wants a date too,
-       so we use today as the date. on python -> postgres there aren't any
-       problems, because postgres simply ignores the extra fields in a date
-       and time string. */
-    time(&now);
-    today = gmtime(&now);
-    memcpy(&tm, today, sizeof(struct tm));
-    
-    Dprintf("psyco_TIME_cast(): s = %s\n", str);
-    
-    n = sscanf(str, "%d:%d:%lf", &hh, &mm, &ss);
-    Dprintf("psyco_TIME_cast(): time parsed, %d components\n", n);
-    Dprintf("psyco_TIME_cast(): hh = %d, mm = %d, ss = %f\n", hh, mm, ss);
-    
-    if (n != 3) {
-        PyErr_SetString(DataError, "unable to parse time");
-        return NULL;
-    }
-    tm.tm_sec = (int)ss;
-    tm.tm_min = mm;
-    tm.tm_hour = hh;
-    
-    return mxDateTimeP->DateTime_FromTmStruct(&tm);
-}
-#else
 static PyObject *
 psyco_TIME_cast(PyObject *s)
 {
     int n, hh=0, mm=0;
     double ss=0.0;
     
-    char *str = PyString_AsString(s);
+    char *str;
 
-    if (s == Py_None) return s;
-    
+    if (s == Py_None) {Py_INCREF(s); return s;}
+    str = PyString_AsString(s);
     /* postgres gives us only the time but a DateTime object wants a date too,
        so we use the unix epoch as the current date. */
     
@@ -156,7 +121,6 @@ psyco_TIME_cast(PyObject *s)
 
     return mxDateTimeP->DateTime_FromDateAndTime(1970, 1, 1, hh, mm ,ss);
 }
-#endif
 
 static char *
 skip_until_space(char *s)
@@ -174,7 +138,7 @@ psyco_INTERVAL_cast(PyObject *s)
     int part = 0;
     char *str;
 
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
     
     str = PyString_AsString(s);
     Dprintf("psyco_INTERVAL_cast(): s = %s\n", str);
@@ -285,7 +249,7 @@ psyco_INTERVAL_cast(PyObject *s)
 static PyObject *
 psyco_BOOLEAN_cast(PyObject *s)
 {
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
     if (PyString_AsString(s)[0] == 't') return PyInt_FromLong(1L);
     else return PyInt_FromLong(0L);
 }
@@ -298,7 +262,7 @@ psyco_BINARY_cast(PyObject *s)
     int len, i;
     PyObject *retstr;
 
-    if (s == Py_None) return s;
+    if (s == Py_None) {Py_INCREF(s); return s;}
    
     str = PyString_AS_STRING(s);
     len = strlen(str);
